==> conc-2.0.1.tar.gz: conc-2.0.1/client/subscribed.c <==
#include "conc.h"
#include "subscribed.h"
#include "groups.h"
#include "error.h"
#include "client-message.h"

#define EDIT_SUBSCRIBED "CONC_edit_subscribed_dialog"

/* 
 * Glade widget names:
 */
#define SUBSCRIBED_DIALOG		"subscribed groups dialog"
#define SUBSCRIBED_DIALOG_TREEVIEW	"treeview"
#define SUBSCRIBED_DIALOG_SUBSCRIBE	"subscribe"
#define SUBSCRIBED_DIALOG_UNSUBSCRIBE	"unsubscribe"

/*
 * Glade handler names:
 */
#define SUBSCRIBED_DIALOG_RESPONSE	"response"
#define SUBSCRIBED_DIALOG_SELECTION	"selection changed"

typedef struct _EditSubscribed {
    Conc *conc;
    GladeXML *xml;
    GtkWindow *window;
    GtkTreeView *treeview;
    GtkButton *subscribe_button;
    GtkButton *unsubscribe_button;
} EditSubscribed;

typedef struct _SubscriptionData {
    EditSubscribed *d;
    gchar *name;
    enum {
	SUBSCRIBE,
	UNSUBSCRIBE
    } action;
} SubscriptionData;

static void
subscribed_selection_changed(GtkTreeView *treeview,
	gpointer user_data) {
    EditSubscribed *d = user_data;
    GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
    GtkTreeIter iter;
    gboolean subscribed;

    if (gtk_tree_selection_get_selected(selection, NULL, &iter) == FALSE) {
	g_object_set(d->subscribe_button, "sensitive", FALSE, NULL);
	g_object_set(d->unsubscribe_button, "sensitive", FALSE, NULL);
	return;
    }
    gtk_tree_model_get(GTK_TREE_MODEL(d->conc->all_groups), &iter,
	    GROUPS_COLUMN_SUBSCRIBED, &subscribed, -1);
    if (subscribed == TRUE) {
	g_object_set(d->subscribe_button, "sensitive", FALSE, NULL);
	g_object_set(d->unsubscribe_button, "sensitive", TRUE, NULL);
    } else {
	g_object_set(d->subscribe_button, "sensitive", TRUE, NULL);
	g_object_set(d->unsubscribe_button, "sensitive", FALSE, NULL);
    }
}

static gchar *
subscribed_selection(EditSubscribed *d) {
    GtkTreeSelection *selection = gtk_tree_view_get_selection(d->treeview);
    GtkTreeIter iter;
    gchar *name = NULL;

    if (gtk_tree_selection_get_selected(selection, NULL, &iter) == FALSE) {
	return NULL;
    }
    gtk_tree_model_get(GTK_TREE_MODEL(d->conc->all_groups), &iter,
	    GROUPS_COLUMN_NAME, &name, -1);
    return name;
}

static void
subscribed_done(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    SubscriptionData *sd = data;
    GError *e = message_arg_error_check(arg);
    GList *consoles = NULL;
    GList *l;
    char *action;
    char *error;

    if (sd->action == SUBSCRIBE) {
	action = "subscribe";
	error = "Could not subscribe";
    } else {
	action = "unsubscribe";
	error = "Could not unsubscribe";
    }
    if (e != NULL) {
	warning_dialog(sd->d->conc, error,
		"Could not %s to group %s (%s).", action, sd->name, e->message);
	g_clear_error(&e);
	g_free(sd->name);
	g_free(sd);
	return;
    }
    switch (sd->action) {
	case SUBSCRIBE:
	    for (l = arg->list; l != NULL; l = g_list_next(l)) {
		MessageArg *a = l->data;

		consoles = g_list_prepend(consoles, a->string->str);
	    }
	    group_add(sd->d->conc, sd->name, consoles);
	    g_list_free(consoles);
	    break;
	case UNSUBSCRIBE:
	    group_delete(sd->d->conc, sd->name);
	    break;
	default:
	    g_assert_not_reached();
    }
    groups_save(sd->d->conc);
    subscribed_selection_changed(sd->d->treeview, sd->d);
    g_free(sd->name);
    g_free(sd);
}

static void
subscribed_subscribe(GtkButton *button,
	gpointer user_data) {
    EditSubscribed *d = user_data;
    gchar *name = subscribed_selection(d);
    SubscriptionData *sd;
    MessageArg *a;
    GError *e = NULL;

    if (name == NULL) {
	return;
    }
    sd = g_new0(SubscriptionData, 1);
    sd->name = name;
    sd->d = d;
    sd->action = SUBSCRIBE;
    a = message_arg_string("cgroup", sd->name);
    if (message_send(d->conc->msg_q, CLIENT_MESSAGE_SUBSCRIBE, a,
		subscribed_done, sd, &e) == FALSE) {
	error_dialog(sd->d->conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}

static void
subscribed_unsubscribe(GtkButton *button,
	gpointer user_data) {
    EditSubscribed *d = user_data;
    gchar *name = subscribed_selection(d);
    SubscriptionData *sd;
    MessageArg *a;
    GError *e = NULL;

    if (name == NULL) {
	return;
    }
    sd = g_new0(SubscriptionData, 1);
    sd->name = name;
    sd->d = d;
    sd->action = UNSUBSCRIBE;
    a = message_arg_string("cgroup", sd->name);
    if (message_send(d->conc->msg_q, CLIENT_MESSAGE_UNSUBSCRIBE, a,
		subscribed_done, sd, &e) == FALSE) {
	error_dialog(sd->d->conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}

static void
cell_toggled(GtkCellRendererToggle *cell,
	gchar *path_string,
	gpointer user_data) {
    EditSubscribed *d = user_data;
    GtkTreeIter iter;
    gchar *name = NULL;
    gboolean subscribed;
    SubscriptionData *sd;
    MessageArg *a;
    GError *e = NULL;
    gchar *command;

    if (gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(d->conc->all_groups),
		&iter, path_string) == FALSE) {
	return;
    }
    gtk_tree_model_get(GTK_TREE_MODEL(d->conc->all_groups), &iter,
	    GROUPS_COLUMN_NAME, &name, 
	    GROUPS_COLUMN_SUBSCRIBED, &subscribed, -1);
    if (name == NULL) {
	return;
    }
    sd = g_new0(SubscriptionData, 1);
    sd->name = name;
    sd->d = d;
    a = message_arg_string("cgroup", sd->name);
    if (subscribed == FALSE) {
	sd->action = SUBSCRIBE;
	command = CLIENT_MESSAGE_SUBSCRIBE;
    } else {
	sd->action = UNSUBSCRIBE;
	command = CLIENT_MESSAGE_UNSUBSCRIBE;
    }
    if (message_send(d->conc->msg_q, command, a,
		subscribed_done, sd, &e) == FALSE) {
	error_dialog(sd->d->conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}


static void
subscribed_close(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    EditSubscribed *d = user_data;

    g_object_set_data(G_OBJECT(d->conc->main_window), EDIT_SUBSCRIBED, NULL);
    gtk_widget_destroy(GTK_WIDGET(d->window));
    g_free(d);
}

extern void
edit_subscribed_show(Conc *conc) {
    EditSubscribed *d = NULL;
    GtkCellRenderer *text_rend, *toggle_rend;
    GtkTreeViewColumn *col;

    d = g_object_get_data(G_OBJECT(conc->main_window), EDIT_SUBSCRIBED);
    if (d != NULL) {
	gtk_window_present(d->window);
	return;
    }
    d = g_new0(EditSubscribed, 1);
    d->conc = conc;
    g_object_set_data(G_OBJECT(conc->main_window), EDIT_SUBSCRIBED, d);
    d->xml = glade_xml_new(conc->glade_file, SUBSCRIBED_DIALOG, NULL);
    d->window = GTK_WINDOW(glade_xml_get_widget(d->xml, SUBSCRIBED_DIALOG));
    d->subscribe_button = GTK_BUTTON(glade_xml_get_widget(d->xml,
		SUBSCRIBED_DIALOG_SUBSCRIBE));
    d->unsubscribe_button = GTK_BUTTON(glade_xml_get_widget(d->xml,
		SUBSCRIBED_DIALOG_UNSUBSCRIBE));
    d->treeview = GTK_TREE_VIEW(glade_xml_get_widget(d->xml,
		SUBSCRIBED_DIALOG_TREEVIEW));
    g_object_set(d->treeview,
	    "model", conc->all_groups,
	    "rules-hint", TRUE,
	    "enable-search", TRUE,
	    "search-column", GROUPS_COLUMN_NAME,
	    NULL);
    text_rend = gtk_cell_renderer_text_new();
    toggle_rend = gtk_cell_renderer_toggle_new();
    col = gtk_tree_view_column_new_with_attributes("Group", text_rend,
	    "text", GROUPS_COLUMN_NAME, NULL);
    gtk_tree_view_append_column(d->treeview, col);
    col = gtk_tree_view_column_new_with_attributes("Subscribed", toggle_rend,
	    "active", GROUPS_COLUMN_SUBSCRIBED, NULL);
    g_object_set(col,
	    "activatable", TRUE,
	    NULL);
    g_signal_connect(toggle_rend, "toggled", (GCallback)cell_toggled, d);
    gtk_tree_view_append_column(d->treeview, col);
    glade_xml_signal_connect_data(d->xml, SUBSCRIBED_DIALOG_SELECTION,
	    (GCallback)subscribed_selection_changed, d);
    glade_xml_signal_connect_data(d->xml, SUBSCRIBED_DIALOG_RESPONSE,
	    (GCallback)subscribed_close, d);
    glade_xml_signal_connect_data(d->xml, SUBSCRIBED_DIALOG_SUBSCRIBE,
	    (GCallback)subscribed_subscribe, d);
    glade_xml_signal_connect_data(d->xml, SUBSCRIBED_DIALOG_UNSUBSCRIBE,
	    (GCallback)subscribed_unsubscribe, d);
}

==> conc-2.0.3.tar.gz: conc-2.0.3/client/subscribed.c <==
#include "conc.h"
#include "subscribed.h"
#include "groups.h"
#include "error.h"
#include "client-message.h"

#define EDIT_SUBSCRIBED "CONC_edit_subscribed_dialog"

/* 
 * Glade widget names:
 */
#define SUBSCRIBED_DIALOG		"subscribed groups dialog"
#define SUBSCRIBED_DIALOG_TREEVIEW	"sgtreeview"
#define SUBSCRIBED_DIALOG_SUBSCRIBE	"sgsubscribe"
#define SUBSCRIBED_DIALOG_UNSUBSCRIBE	"sgunsubscribe"

/*
 * Glade handler names:
 */
#define SUBSCRIBED_DIALOG_RESPONSE	"response"
#define SUBSCRIBED_DIALOG_SELECTION	"selection changed"
#define SUBSCRIBED_DIALOG_SUB		"subscribe"
#define SUBSCRIBED_DIALOG_UNSUB		"unsubscribe"

typedef struct _EditSubscribed {
    Conc *conc;
    GladeXML *xml;
    GtkWindow *window;
    GtkTreeView *treeview;
    GtkButton *subscribe_button;
    GtkButton *unsubscribe_button;
} EditSubscribed;

typedef struct _SubscriptionData {
    EditSubscribed *d;
    gchar *name;
    enum {
	SUBSCRIBE,
	UNSUBSCRIBE
    } action;
} SubscriptionData;

static void
subscribed_selection_changed(GtkTreeView *treeview,
	gpointer user_data) {
    EditSubscribed *d = user_data;
    GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
    GtkTreeIter iter;
    gboolean subscribed;

    if (gtk_tree_selection_get_selected(selection, NULL, &iter) == FALSE) {
	g_object_set(d->subscribe_button, "sensitive", FALSE, NULL);
	g_object_set(d->unsubscribe_button, "sensitive", FALSE, NULL);
	return;
    }
    gtk_tree_model_get(GTK_TREE_MODEL(d->conc->all_groups), &iter,
	    GROUPS_COLUMN_SUBSCRIBED, &subscribed, -1);
    if (subscribed == TRUE) {
	g_object_set(d->subscribe_button, "sensitive", FALSE, NULL);
	g_object_set(d->unsubscribe_button, "sensitive", TRUE, NULL);
    } else {
	g_object_set(d->subscribe_button, "sensitive", TRUE, NULL);
	g_object_set(d->unsubscribe_button, "sensitive", FALSE, NULL);
    }
}

static gchar *
subscribed_selection(EditSubscribed *d) {
    GtkTreeSelection *selection = gtk_tree_view_get_selection(d->treeview);
    GtkTreeIter iter;
    gchar *name = NULL;

    if (gtk_tree_selection_get_selected(selection, NULL, &iter) == FALSE) {
	return NULL;
    }
    gtk_tree_model_get(GTK_TREE_MODEL(d->conc->all_groups), &iter,
	    GROUPS_COLUMN_NAME, &name, -1);
    return name;
}

static void
subscribed_done(MessageQueue q,
	const MessageArg *arg,
	gpointer data) {
    SubscriptionData *sd = data;
    GError *e = message_arg_error_check(arg);
    GList *consoles = NULL;
    GList *l;
    char *action;
    char *error;

    if (sd->action == SUBSCRIBE) {
	action = "subscribe";
	error = "Could not subscribe";
    } else {
	action = "unsubscribe";
	error = "Could not unsubscribe";
    }
    if (e != NULL) {
	warning_dialog(sd->d->conc, error,
		"Could not %s to group %s (%s).", action, sd->name, e->message);
	g_clear_error(&e);
	g_free(sd->name);
	g_free(sd);
	return;
    }
    switch (sd->action) {
	case SUBSCRIBE:
	    for (l = arg->list; l != NULL; l = g_list_next(l)) {
		MessageArg *a = l->data;

		consoles = g_list_prepend(consoles, a->string->str);
	    }
	    group_add(sd->d->conc, sd->name, consoles);
	    g_list_free(consoles);
	    break;
	case UNSUBSCRIBE:
	    group_delete(sd->d->conc, sd->name);
	    break;
	default:
	    g_assert_not_reached();
    }
    groups_save(sd->d->conc);
    subscribed_selection_changed(sd->d->treeview, sd->d);
    g_free(sd->name);
    g_free(sd);
}

static void
subscribed_subscribe(GtkButton *button,
	gpointer user_data) {
    EditSubscribed *d = user_data;
    gchar *name = subscribed_selection(d);
    SubscriptionData *sd;
    MessageArg *a;
    GError *e = NULL;

    if (name == NULL) {
	return;
    }
    sd = g_new0(SubscriptionData, 1);
    sd->name = name;
    sd->d = d;
    sd->action = SUBSCRIBE;
    a = message_arg_string("cgroup", sd->name);
    if (message_send(d->conc->msg_q, CLIENT_MESSAGE_SUBSCRIBE, a,
		subscribed_done, sd, &e) == FALSE) {
	error_dialog(sd->d->conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}

static void
subscribed_unsubscribe(GtkButton *button,
	gpointer user_data) {
    EditSubscribed *d = user_data;
    gchar *name = subscribed_selection(d);
    SubscriptionData *sd;
    MessageArg *a;
    GError *e = NULL;

    if (name == NULL) {
	return;
    }
    sd = g_new0(SubscriptionData, 1);
    sd->name = name;
    sd->d = d;
    sd->action = UNSUBSCRIBE;
    a = message_arg_string("cgroup", sd->name);
    if (message_send(d->conc->msg_q, CLIENT_MESSAGE_UNSUBSCRIBE, a,
		subscribed_done, sd, &e) == FALSE) {
	error_dialog(sd->d->conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}

static void
cell_toggled(GtkCellRendererToggle *cell,
	gchar *path_string,
	gpointer user_data) {
    EditSubscribed *d = user_data;
    GtkTreeIter iter;
    gchar *name = NULL;
    gboolean subscribed;
    SubscriptionData *sd;
    MessageArg *a;
    GError *e = NULL;
    gchar *command;

    if (gtk_tree_model_get_iter_from_string(GTK_TREE_MODEL(d->conc->all_groups),
		&iter, path_string) == FALSE) {
	return;
    }
    gtk_tree_model_get(GTK_TREE_MODEL(d->conc->all_groups), &iter,
	    GROUPS_COLUMN_NAME, &name, 
	    GROUPS_COLUMN_SUBSCRIBED, &subscribed, -1);
    if (name == NULL) {
	return;
    }
    sd = g_new0(SubscriptionData, 1);
    sd->name = name;
    sd->d = d;
    a = message_arg_string("cgroup", sd->name);
    if (subscribed == FALSE) {
	sd->action = SUBSCRIBE;
	command = CLIENT_MESSAGE_SUBSCRIBE;
    } else {
	sd->action = UNSUBSCRIBE;
	command = CLIENT_MESSAGE_UNSUBSCRIBE;
    }
    if (message_send(d->conc->msg_q, command, a,
		subscribed_done, sd, &e) == FALSE) {
	error_dialog(sd->d->conc, _("Internal error"),
		_("Failed to send message (%s)"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}


static void
subscribed_close(GtkDialog *dialog,
	gint arg1,
	gpointer user_data) {
    EditSubscribed *d = user_data;

    g_object_set_data(G_OBJECT(d->conc->main_window), EDIT_SUBSCRIBED, NULL);
    gtk_widget_destroy(GTK_WIDGET(d->window));
    g_free(d);
}

extern void
edit_subscribed_show(Conc *conc) {
    EditSubscribed *d = NULL;
    GtkCellRenderer *text_rend, *toggle_rend;
    GtkTreeViewColumn *col;

    d = g_object_get_data(G_OBJECT(conc->main_window), EDIT_SUBSCRIBED);
    if (d != NULL) {
	gtk_window_present(d->window);
	return;
    }
    d = g_new0(EditSubscribed, 1);
    d->conc = conc;
    g_object_set_data(G_OBJECT(conc->main_window), EDIT_SUBSCRIBED, d);
    d->xml = glade_xml_new(conc->glade_file, SUBSCRIBED_DIALOG, NULL);
    d->window = GTK_WINDOW(glade_xml_get_widget(d->xml, SUBSCRIBED_DIALOG));
    d->subscribe_button = GTK_BUTTON(glade_xml_get_widget(d->xml,
		SUBSCRIBED_DIALOG_SUBSCRIBE));
    d->unsubscribe_button = GTK_BUTTON(glade_xml_get_widget(d->xml,
		SUBSCRIBED_DIALOG_UNSUBSCRIBE));
    d->treeview = GTK_TREE_VIEW(glade_xml_get_widget(d->xml,
		SUBSCRIBED_DIALOG_TREEVIEW));
    g_object_set(d->treeview,
	    "model", conc->all_groups,
	    "rules-hint", TRUE,
	    "enable-search", TRUE,
	    "search-column", GROUPS_COLUMN_NAME,
	    NULL);
    text_rend = gtk_cell_renderer_text_new();
    toggle_rend = gtk_cell_renderer_toggle_new();
    col = gtk_tree_view_column_new_with_attributes("Group", text_rend,
	    "text", GROUPS_COLUMN_NAME, NULL);
    gtk_tree_view_append_column(d->treeview, col);
    col = gtk_tree_view_column_new_with_attributes("Subscribed", toggle_rend,
	    "active", GROUPS_COLUMN_SUBSCRIBED, NULL);
    g_object_set(col,
	    "activatable", TRUE,
	    NULL);
    g_signal_connect(toggle_rend, "toggled", (GCallback)cell_toggled, d);
    gtk_tree_view_append_column(d->treeview, col);
    glade_xml_signal_connect_data(d->xml, SUBSCRIBED_DIALOG_SELECTION,
	    (GCallback)subscribed_selection_changed, d);
    glade_xml_signal_connect_data(d->xml, SUBSCRIBED_DIALOG_RESPONSE,
	    (GCallback)subscribed_close, d);
    glade_xml_signal_connect_data(d->xml, SUBSCRIBED_DIALOG_SUB,
	    (GCallback)subscribed_subscribe, d);
    glade_xml_signal_connect_data(d->xml, SUBSCRIBED_DIALOG_UNSUB,
	    (GCallback)subscribed_unsubscribe, d);
}
